<?php
/**
 * A file i/o iterator
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2012 Hub Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class FileIoIterator extends BaseIterator implements SeekableWritableFileIterator {
	/**
	 * Back-buffer
	 */
	private $backBuffer = '';

	/**
	 * Currently loaded block (will be returned by current())
	 */
	private $currentBlock = '';

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @param	$pointerInstance	An instance of a InputOutputPointer class
	 * @param	$blockInstance		An instance of a CalculatableBlock class
	 * @return	$iteratorInstance	An instance of a Iterator class
	 */
	public final static function createFileIoIterator (InputOutputPointer $pointerInstance, CalculatableBlock $blockInstance) {
		// Get new instance
		$iteratorInstance = new FileIoIterator();

		// Set the instance here
		$iteratorInstance->setPointerInstance($pointerInstance);

		// Set the block instance here
		$iteratorInstance->setBlockInstance($blockInstance);

		// Return the prepared instance
		return $iteratorInstance;
	}

	/**
	 * Initializes the back-buffer by setting it to an empty string.
	 *
	 * @return	void
	 */
	private function initBackBuffer () {
		// Simply call the setter
		$this->setBackBuffer('');
	}

	/**
	 * Setter for backBuffer field
	 *
	 * @param	$backBuffer		Characters to "store" in back-buffer
	 * @return	void
	 */
	private function setBackBuffer ($backBuffer) {
		// Cast to string (so no arrays or objects)
		$backBuffer = (string) $backBuffer;

		// ... and set it
		$this->backBuffer = $backBuffer;
	}

	/**
	 * Getter for backBuffer field
	 *
	 * @return	$backBuffer		Characters "stored" in back-buffer
	 */
	private function getBackBuffer () {
		return $this->backBuffer;
	}

	/**
	 * Setter for currentBlock field
	 *
	 * @param	$currentBlock		Characters to set a currently loaded block
	 * @return	void
	 */
	private function setCurrentBlock ($currentBlock) {
		// Cast to string (so no arrays or objects)
		$currentBlock = (string) $currentBlock;

		// ... and set it
		$this->currentBlock = $currentBlock;
	}

	/**
	 * Gets currently read data
	 *
	 * @return	$current	Currently read data
	 */
	public function current () {
		// Return it
		return $this->currentBlock;
	}

	/**
	 * Gets current seek position ("key").
	 *
	 * @return	$key	Current key in iteration
	 */
	public function key () {
		// Return it
		return $this->getPointerInstance()->determineSeekPosition();
	}

	/**
	 * Advances to next "block" of bytes
	 *
	 * @return	void
	 */
	public function next () {
		// Is there nothing to read?
		if (!$this->valid()) {
			// Nothing to read
			return;
		} // END - if

		// First calculate minimum block length
		$length = $this->getBlockInstance()->caluclateMinimumBlockLength();

		// Short be more than zero!
		assert($length > 0);

		// Wait until a entry/block separator has been found
		$data = $this->getBackBuffer();
		while (($this->getPointerInstance()->isEndOfFileReached()) && (!$this->getBlockInstance()->isBlockSeparatorFound($data))) {
			// Then read the block
			$data .= $this->read($length);
			/* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput('data()=' . strlen($data));
		} // END - if

		// EOF reached?
		if ($this->getPointerInstance()->isEndOfFileReached()) {
			// Then abort here silently
			return;
		} // END - if

		/*
		 * Init back-buffer which is the data that has been found beyond the
		 * separator.
		 */
		$this->initBackBuffer();

		// Separate data
		$dataArray = explode(self::getBlockSeparator(), $data);

		// Left part is the actual block, right one the back-buffer data
		$this->setCurrentBlock($dataArray[0]);
		$this->setBackBuffer($dataArray[1]);
	}

	/**
	 * Rewinds to the beginning of the file
	 *
	 * @return	$status		Status of this operation
	 */
	public function rewind () {
		// Call pointer instance
		return $this->getPointerInstance()->rewind();
	}

	/**
	 * Checks wether the current entry is valid (not at the end of the file).
	 * This method will return TRUE if an emptied (nulled) entry has been found.
	 *
	 * @return	$isValid	Whether the next entry is valid
	 */
	public function valid () {
		// First calculate minimum block length
		$length = $this->getBlockInstance()->caluclateMinimumBlockLength();

		// Short be more than zero!
		assert($length > 0);

		// Get current seek position
		$seekPosition = $this->key();

		// Then try to read it
		$data = $this->read($length);

		// If some bytes could be read, all is fine
		$isValid = ((is_string($data)) && (strlen($data) > 0));

		// Get header size
		$headerSize = $this->getBlockInstance()->getHeaderSize();

		// Is the seek position at or beyond the header?
		if ($seekPosition >= $headerSize) {
			// Seek back to old position
			$this->seek($seekPosition);
		} else {
			// Seek directly behind the header
			$this->seek($headerSize);
		}

		// Return result
		return $isValid;
	}

	/**
	 * Seeks to given position
	 *
	 * @param	$seekPosition	Seek position in file
	 * @return	$status			Status of this operation
	 */
	public function seek ($seekPosition) {
		// Call pointer instance
		return $this->getPointerInstance()->seek($seekPosition);
	}

	/**
	 * Writes at given position by seeking to it.
	 *
	 * @param	$seekPosition	Seek position in file
	 * @param	$data			Data to be written
	 * @return	void
	 */
	public function writeAtPosition ($seekPosition, $data) {
		// First seek to it
		$this->seek($seekPosition);

		// Then write the data at that position
		$this->getPointerInstance()->writeToFile($data);
	}

	/**
	 * Size of file stack
	 *
	 * @return	$size	Size (in bytes) of file
	 */
	public function size () {
		// Call the pointer object
		$size = $this->getPointerInstance()->size();

		// Return result
		return $size;
	}

	/**
	 * Reads given amount of bytes from file.
	 *
	 * @param	$bytes	Amount of bytes to read
	 * @return	$data	Data read from file
	 */
	public function read ($bytes) {
		// Call pointer instance
		return $this->getPointerInstance()->read($bytes);
	}
}

// [EOF]
?>
